
// m3p.cpp - M3P state from edge marginals, via spanning tree
// (c) 2008, 2009 Nic Schraudolph - see COPYRIGHT.txt for details

#include <fstream>
#include "graph.hpp"

int main(int argc, char** argv)
{
    if (argc != 2)
    {
        std::cerr << "usage: " << argv[0] << " <edgelist> "
            "< <probs> > <nodestates>" << std::endl;
        exit(1);
    }
    Isinf::argv0 = argv[0];

    // read edge list from first argument
    Isinf::Graph g;
    std::ifstream arg1(argv[1]);
    if (arg1.fail())
    {
        std::cerr << Isinf::argv0 << "cannot open file '" << argv[1] << "'\n";
        exit(2);
    }

    std::vector<size_t> x;
    arg1 >> x;
    while (x.size() >= 2)
    {
        if (!g.add_edge(x[0], x[1]))
        {
            std::cerr << Isinf::argv0 << "bad edge ("
                << x[0] << ',' << x[1] << ')' << std::endl;
            exit(3);
        }
        x.clear();
        arg1 >> x;
    }
    assert(g.is_connected());
    
    // read marginals from standard input
    std::vector<double> p;
    while (std::cin.good())
    {
        std::cin >> p;
        if (p.empty()) continue;

        if (p.size() != g.nedges)
        {
            std::cerr << Isinf::argv0 << "read " << p.size()
                << " values instead of " << g.nedges << std::endl;
            exit(4);
        }

        for (size_t i = 0; i < p.size(); ++i)
        {
            p[i] = 0.5 - p[i];
            g.edges[i]->agree = g.edges[i]->other->agree = (p[i] > 0.0);
        }
        Isinf::CPUtimer *t = new Isinf::CPUtimer;
        g.mmc_nodestate(p);
        delete t;
        p.clear();

        for (size_t i = 0; i < g.nodes.size(); ++i)
            std::cout << (g.nodes[i].state ? '+' : '-');
        std::cout << ' ' << std::endl;
    }
}

